'use strict';

const etools=require('../east-tools');

/**
 * 将proto对象转json方法
 */
function protoObj2jsonObj(obj){
  switch (etools.protoType(obj)) {
    case '[object Null]':
    case '[object RegExp]':
    case '[object Uint8Array]':
    case '[object Undefined]':
    case '[object Boolean]':
    case '[object Number]':
    case '[object String]': return obj;
    case '[object Array]': return obj.map(i=>{return protoObj2jsonObj(i)});
    case '[object Object]':
      // 本就是jsonObj，原样返回
      if(obj.constructor.name==='Object') return obj;

      // Long处理
      if(obj.constructor.name==='Long') return obj.toNumber();

      // 构造新jsonObj
      const jsonObj={
        _protoBufType:obj.constructor.name
      };
      // 遍历赋值
      for(const key in obj){
        if(!obj.hasOwnProperty(key)) continue;
        jsonObj[key]=protoObj2jsonObj(obj[key]);
      }

      // 反馈
      return jsonObj;
  }
}

const netPoint  = require('./c_netPoint');
const protoWork = require('./protoWork');

// 对外输出
module.exports={
  isDebug:require('./isDebug'),
  c_fluentCache:require('./c_fluentCache'),
  c_apiConn:require('./c_apiConn'),
  c_econn:require('./c_econn'),
  c_apiFunc:require('./c_apiFunc'),
  c_event:require('./c_event'),
  c_master:netPoint.c_master,
  c_service:netPoint.c_service,
  enetFuncs:netPoint.enetFuncs,
  protoWork,protoObj2jsonObj,
  // 添加proto文件
  addProtoFile(filepath){
    protoWork.addProtoFile(filepath)
  }
};
