
const protobufjs=require('protobufjs');
const etools=require('../east-tools');

const protoBuf  =etools.isNode? // 根据前端还是后端决定如何读取
  protobufjs.loadSync(__dirname+'/Plant3.proto'):     // 后端可以直接读取proto文件
  protobufjs.Root.fromJSON(require('./Plant3.json')); // 前端需读取Plant3.json文件

class c_protoWork{
  constructor(root){
    this._root=root;
  }

  /**
   * 将json序列化为指定格式
   * @param json 必须是字典
   * @param typ 指定的格式字符串
   */
  encode(json,typ){
    // json必须是字典
    if(etools.etype(json)!=='dict') throw new Error('json is not Dict');
    // 预备proto格式
    const protoObj=this._root.lookupType(typ);
    // 确认格式
    const errMsg=protoObj.verify(json);
    if(errMsg) throw new Error('json format Error with type: '+typ+'  Json: '+JSON.stringify(json));
    // 实例化消息
    const mid=protoObj.create(json);
    // 取得流并反馈
    return protoObj.encode(mid).finish();
  }

  decode(buf,typ){
    // 预备proto格式
    const protoObj=this._root.lookupType(typ);
    // 解析并反馈
    return protoObj.decode(buf);
  }

  valueEncode(obj,pArry){
    // 防嵌套数组，递归时传递，没有则新建
    if(!pArry) pArry=[];

    // 获取原型类别
    const objTyp=etools.protoType(obj);

    // 根据etype递归调用
    switch (objTyp) {
      case '[object Undefined]':return {typ:'0'};
      case '[object Null]':return {typ:'1'};
      case '[object Number]':
        if(Number.isSafeInteger(obj)) return {typ:'2',raw:this.encode({obj},'ProtoBuf.Plant3.v_int')};
        else return {typ:'3',raw:this.encode({obj},'ProtoBuf.Plant3.v_float')};
      case '[object Array]':
        const array=new Array(obj.length);
        for(let i=0;i<obj.length;i++)
          array[i]=this.valueEncode(obj[i],pArry);
        return {typ:'4',raw:this.encode({obj:array},'ProtoBuf.Plant3.v_array')};
      // buffer
      case '[object Uint8Array]':
        return {typ:'5',raw:this.encode({obj},'ProtoBuf.Plant3.v_bytes')};
      case '[object String]':
        return {typ:'6',raw:this.encode({obj},'ProtoBuf.Plant3.v_string')};
      case '[object Object]':
        // 类别优先使用指定的，若没有则使用原型名
        const protoTyp=obj._protoBufType||obj.constructor.name;
        // 普通对象
        if(protoTyp==='Object'){
          // 嵌套检查，有嵌套则报错
          if(pArry.includes(obj)) throw new Error('obj 嵌套错误');
          pArry.push(obj);

          // 递归调用
          const dictArry=[];
          for(const key in obj){
            // 忽略 __proto__项目
            if(key==='__proto__') continue;
            dictArry.push({key,val:this.valueEncode(obj[key],pArry)});
          }
          return {typ:'7',raw:this.encode({obj:dictArry},'ProtoBuf.Plant3.v_dict')};
        }
        // ProtoBuf对象
        else {
          // 不能删除 delete obj._protoBufType;
          return {typ:protoTyp,raw:this.encode(obj,protoTyp)};
        }
      case '[object Boolean]':
        // true false 分别占用一个单独的类型代码
        return {typ:obj?'8':'9'};
      default:
        throw new Error('unsport format: '+protoTyp);
    }
  }

  valueDecode(paramItem){
    switch (paramItem.typ){
      case '0':return;// undefined
      case '1':return null;
      case '8':return true;
      case '9':return false;
      case '2':
        const obj=this.decode(paramItem.raw,'ProtoBuf.Plant3.v_int').obj;
        // 后台需要toNumber方法，前端不能执行
        return (typeof obj==='number')?obj:obj.toNumber();
      case '3':
        return this.decode(paramItem.raw,'ProtoBuf.Plant3.v_float').obj;
      case '4':
        // 先提取成数组
        const arry=this.decode(paramItem.raw,'ProtoBuf.Plant3.v_array').obj;
        // 再迭代还原
        return arry.map(r=>{return this.valueDecode(r)});
      case '5':
        return this.decode(paramItem.raw,'ProtoBuf.Plant3.v_bytes').obj;
      case '6':
        return this.decode(paramItem.raw,'ProtoBuf.Plant3.v_string').obj;
      case '7':
        // 先提取成数组
        const dictArry=this.decode(paramItem.raw,'ProtoBuf.Plant3.v_dict').obj;
        const result={};
        dictArry.forEach(r=>result[r.key]=this.valueDecode(r.val));
        return result;
      default:
        return this.decode(paramItem.raw,paramItem.typ);
    }
  }

  lookupType(typ){
    return this._root.lookupType(typ);
  }

  addProtoFile(filepath){
    this._root.loadSync(filepath);
  }
}
const protoWork=new c_protoWork(protoBuf);

module.exports=protoWork;